@@ -3,6 +3,8 @@ from toolbox.utils.format_data_alphalens import price_format_for_alphalens, fact
 from .ml.ml_factor_calculation import calc_ml_factor
 from .ml.model_wrapper import ModelWrapper
 from .db.read.query_constructor import QueryConstructor
+from .db.api.sql_connection import SQLConnection
+from .db.read.db_functions import table_info
 
 __all__ = [
     'ConstituteAdjustment',
@@ -10,6 +12,8 @@ __all__ = [
     'factor_format_for_alphalens',
     'calc_ml_factor',
     'ModelWrapper',
-    'QueryConstructor'
+    'QueryConstructor',
+    'SQLConnection',
+    'table_info'
 ]
 
