@@ -3,6 +3,7 @@ Define Global Settings
 """
 
 DB_CONNECTION_STRING = '/Users/alex/Desktop/DB/wrds.duckdb'  # the directory to the sql database
+CACHE_DIRECTORY = '/tmp/toolbox_cache'  # the directory to cache files
 
 DB_ADJUSTOR_FIELDS = {
     'cstat.security_daily': [
@@ -32,5 +33,8 @@ DB_ADJUSTOR_FIELDS = {
 
     'wrds.firm_ratios': [
         {'fields_to_adjust': []}
+    ],
+    'ibes.summary_price_target': [
+        {'fields_to_adjust': []}
     ]
 }
